@@ -3,6 +3,10 @@
  *  All Rights Reserved.
  */
 
+#define MOST_STATUS_COLOR	1
+#define MOST_ULINE_COLOR	2
+#define MOST_BOLD_COLOR		3
+
 extern void most_tt_reverse_video(void);
 extern void most_tt_bold_video(void);
 extern void most_tt_underline_video(void);
