@@ -1,7 +1,7 @@
 /*
  This file is part of MOST.
 
- Copyright (c) 1991, 1999 John E. Davis
+ Copyright (c) 1991, 1999, 2002, 2005 John E. Davis
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -17,7 +17,7 @@
  with this program; if not, write to the Free Software Foundation, Inc., 675
  Mass Ave, Cambridge, MA 02139, USA. 
 */
-#define MOST_VERSION "4.9.2"
+#define MOST_VERSION "4.10.2"
 
 #include "config.h"
 
@@ -64,6 +64,7 @@ int Most_Captive_Mode;
 #if MOST_HAS_MMAP
 int Most_Disable_MMap	= 0;
 #endif
+int Most_UTF8_Mode = 0;
 
 static int  Most_Starting_Line;
 char *Most_Program;	/* Program Name (argv[0]) */
@@ -79,7 +80,12 @@ char *Most_Version = MOST_VERSION;
 
 void most_usage (void)
 {
-   fprintf(stderr,"MOST version %s Usage:\n", Most_Version);
+   fprintf(stderr,"MOST version %s (S-Lang version %s)\n",
+	   Most_Version, SLang_Version_String);
+   if (SLang_Version != SLANG_VERSION)
+     fprintf (stderr, " *Note: This executable was compiled against S-Lang %s\n", SLANG_VERSION_STRING);
+     
+   fprintf (stderr, "Usage:\n");
    fprintf(stderr, "most [-1Cbckstvw] [+/string] [+line number] [+s] [+d] file...\n");
    fputs(" where: -1:  assume VT100 terminal. (VMS only)\n", stderr);
    fputs("        -b:  Startup in binary mode.\n", stderr);
@@ -433,7 +439,10 @@ int most (int argc, char **argv)
 	  }
      }
    else
-     strcpy(file,argv[i]);
+     {
+	strncpy (file, argv[i], sizeof(file));
+	file[sizeof(file)-1] = 0;
+     }
 
    if (!isatty(fileno(stdout)))
      {
@@ -445,6 +454,15 @@ int most (int argc, char **argv)
    context = 0;
 
    SLtt_get_terminfo();
+#if SLANG_VERSION >= 20000
+#if 0
+   Most_UTF8_Mode = SLutf8_enable (1);
+   if (Most_UTF8_Mode)
+     {
+	fprintf (stderr, "UTF-8 Mode is in effect\n");
+     }
+#endif
+#endif
    SLtt_Ignore_Beep = 1;
    if (No_Colors) 
      SLtt_Use_Ansi_Colors = 0;
@@ -498,3 +516,18 @@ int most (int argc, char **argv)
    most_exit_most ();
    return status;
 }
+
+#if SLANG_VERSION <= 10409
+
+int SLang_set_error (int x)
+{
+   SLang_Error = x;
+   return 0;
+}
+
+int SLang_get_error (void)
+{
+   return SLang_Error;
+}
+
+#endif
