@@ -14,6 +14,8 @@ extern int Most_Z_Opt;             /* gunzip-on-the-fly */
 extern int Most_Disable_MMap;
 #endif
 
+extern int Most_UTF8_Mode;
+
 extern int Most_Secure_Mode;
 extern int Most_Captive_Mode;
 extern char *Most_Program;
@@ -40,7 +42,15 @@ extern int Most_Tab_Width;
 #endif
 
 #ifdef HAVE_SNPRINTF
-# define _SLsnprintf snprintf
+# define _pSLsnprintf snprintf
 #else
-extern int _SLsnprintf (char *, unsigned int, char *, ...);
+# if SLANG_VERSION < 20000
+#  define _pSLsnprintf _SLsnprintf
+# endif
+extern int _pSLsnprintf (char *, unsigned int, char *, ...);
+#endif
+
+#if SLANG_VERSION <= 10409
+extern int SLang_get_error (void);
+extern int SLang_set_error (int);
 #endif
