@@ -1,7 +1,7 @@
 /*
  This file is part of MOST.
 
- Copyright (c) 1991, 1999, 2002 John E. Davis
+ Copyright (c) 1991, 1999, 2002, 2005 John E. Davis
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -113,7 +113,8 @@ void most_put_message ()
 	if (SLang_input_pending (5))
 	  {
 	     (void) SLang_getkey ();
-	     SLang_Error = SLKeyBoard_Quit = 0;
+	     SLKeyBoard_Quit = 0;
+	     SLang_set_error (0);
 	  }
      }
    Beep_Mini = 0;
@@ -125,6 +126,9 @@ void most_put_message ()
 
 SLang_RLine_Info_Type *Most_RLI;
 
+#if SLANG_VERSION < 20000
+typedef SLang_RLine_Info_Type SLrline_Type;
+
 static void rline_update (unsigned char *buf, int len, int col)
 {
    SLsmg_gotorc (SLtt_Screen_Rows - 1, 0);
@@ -133,7 +137,34 @@ static void rline_update (unsigned char *buf, int len, int col)
    SLsmg_gotorc (SLtt_Screen_Rows - 1, col);
    SLsmg_refresh ();
 }
+#else
+static void rline_update (SLrline_Type *rli, char *prompt, 
+			  char *buf, unsigned int len, unsigned int point,
+			  VOID_STAR client_data)
+{
+   int col;
 
+   (void) client_data;
+   (void) rli;
+   
+   while (1)
+     {
+	SLsmg_gotorc (SLtt_Screen_Rows - 1, 0);
+	SLsmg_write_string (prompt);
+	SLsmg_write_nchars (buf, point);
+	col = SLsmg_get_column ();
+	SLsmg_write_nchars (buf + point, len - point);
+	if (col < SLtt_Screen_Cols)
+	  break;
+	buf++; point--; len--;      /* FIXME for UTF-8 */
+     }
+   SLsmg_erase_eol ();
+   SLsmg_gotorc (SLtt_Screen_Rows - 1, col);
+   SLsmg_refresh ();
+}
+#endif
+
+#if SLANG_VERSION < 20000
 static SLang_RLine_Info_Type  *init_readline (void)
 {
    unsigned char *buf = NULL;
@@ -143,7 +174,7 @@ static SLang_RLine_Info_Type  *init_readline (void)
        || (NULL == (buf = (unsigned char *) SLMALLOC (256))))
      {
 	fprintf(stderr, "malloc error.\n");
-	exit(-1);
+	exit(11);
      }
 
    SLMEMSET ((char *) rli, 0, sizeof (SLang_RLine_Info_Type));
@@ -163,36 +194,91 @@ static SLang_RLine_Info_Type  *init_readline (void)
    return rli;
 }
 
-int most_read_from_minibuffer(char *prompt, char *what)
+static char *SLrline_read_line (SLrline_Type *rli, char *prompt, unsigned int *lenp)
 {
    int i;
+   
+   rli->prompt = prompt;
+   i = SLang_read_line (rli);
+   if (i < 0)
+     {
+	*lenp = 0;
+	return NULL;
+     }
+   *lenp = i;
+   return SLmake_string ((char *)rli->buf);
+}
+
+static int SLrline_set_line (SLrline_Type *rli, char *what)
+{
+   strcpy ((char *)rli->buf, what);
+   return 0;
+}
+#define SLrline_save_line SLang_rline_save_line
+
+static int SLrline_set_point (SLrline_Type *rli, int point)
+{
+   rli->point = point;
+   return 0;
+}
+
+#else
+static SLang_RLine_Info_Type  *init_readline (void)
+{
+   SLang_RLine_Info_Type *rli;
+   unsigned int flags = SL_RLINE_BLINK_MATCH;
+   
+   if (NULL == (rli = SLrline_open (SLtt_Screen_Cols, flags)))
+     return NULL;
+
+   (void) SLrline_set_update_hook (rli, rline_update, NULL);
+   return rli;
+}
+#endif
+
+int most_read_from_minibuffer(char *prompt, char *what, unsigned int buflen)
+{
+   int i = 0;
+   char *buf;
+   unsigned int len;
 
    if (Most_RLI == NULL)
      {
 	Most_RLI = init_readline ();
      }
-
+#if SLANG_VERSION < 20000
    Most_RLI->edit_width = SLtt_Screen_Cols - 1;
    Most_RLI->prompt = prompt;
    *Most_RLI->buf = 0;
+#else
+   SLrline_set_display_width (Most_RLI, SLtt_Screen_Cols);
+#endif
+
    most_select_minibuffer();
 
    /* do not use default.  The up arrow can always get it back. */
-   if ((*what) && (what != Most_Search_Str))
+   if ((what != NULL) 
+       && (*what) && (what != Most_Search_Str))
      {
-	strcpy ((char *) Most_RLI->buf, what);
-	Most_RLI->point = strlen (what);
+	if (-1 == SLrline_set_line (Most_RLI, what))
+	  return -1;
+
+	SLrline_set_point (Most_RLI, strlen (what));
      }
 
-   /* next routine returns -1 upon quit */
-   i = SLang_read_line (Most_RLI);
-   if ((i > 0) && !SLang_Error && !SLKeyBoard_Quit)
+   buf = SLrline_read_line (Most_RLI, prompt, &len);
+
+   if ((buf != NULL) && (0 == SLang_get_error()) && !SLKeyBoard_Quit)
      {
-	SLang_rline_save_line (Most_RLI);
-	strcpy(what, (char *) Most_RLI->buf);
+	SLrline_save_line (Most_RLI);
+	strncpy(what, (char *) buf, buflen);
+	what[buflen-1] = 0;
      }
+   SLfree (buf);
+
    if (SLKeyBoard_Quit) i = -1;
-   SLang_Error = SLKeyBoard_Quit = 0;
+   SLang_set_error (0);
+   SLKeyBoard_Quit = 0;
 
    most_exit_minibuffer();
 
